/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2012 digitalSTROM.org, Zurich, Switzerland
 *  Author: Roman K�hler
 */



function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}

function getStateGridTextActive (scene)
{
    var scene_name = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextActive = Ext.String.format2(_("only when {name} is active"), {'name': scene_name});
    return gridTextActive;
}

function getStateGridTextInactive (scene)
{
    var scene_name = dss.staticDataModel.getNameForScene(0, 0, scene);
    var gridTextInactive = Ext.String.format2(_("only when {name} is inactive"), {'name': scene_name});
    return gridTextInactive;
}


Ext.define('DSS.component.checkPosColumn', {
    extend: 'Ext.grid.column.Column',
    alias: 'widget.phonecolumn',
    
    constructor: function() {
        this.addEvents('checkPhoneChange');
        this.callParent(arguments);
    },

    processEvent: function(type, view, cell, recordIndex, cellIndex, e) {
        if (type == 'mousedown') {
            var record = view.panel.store.getAt(recordIndex);
            
            
            var dataIndex = this.dataIndex;
            var checked = !record.get(dataIndex);
                
            record.set(dataIndex, checked);
            this.fireEvent('checkPhoneChange', this, record, recordIndex, checked);
            return this.callParent(arguments);
        }
    },

    renderer : function(val, metadata, record, rowIdx, colIdx, store, view){
        
        if (val) {
            return '<div style="margin:0 auto;width:13px;height:13px;background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif); background-position: 0px -13px;" />';
        } else {
            return '<div style="margin:0 auto;width:13px;height:13px;background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif);" />';            
        }
    }
});

Ext.define('DSS.addon.UDSEdit', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : 'User defined states',
        appId : 'UDS',
        appVersion : '1.0.0.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },
    getContent : function() {
        var me=this;
        this.activityStateEditor = Ext.create('DSS.editor.ActivityStateEditorPart', {});
        this.nameEditor = Ext.create('DSS.editor.CombinedNamesControl',{});
        this.combinedStateEditor = Ext.create('DSS.editor.CombinedStateEditorPart', {});
        this.sensorStateEditor = Ext.create('DSS.editor.SensorStateEditorPart', {});
        this.consumptionStateEditor = Ext.create('DSS.editor.ConsumptionStateEditorPart', {});
        this.windowStateEditor = Ext.create('DSS.editor.WindowStateEditorPart', {});
        this.starterStateEditor=Ext.create('DSS.editor.StarterEditorPart',{});
        this.powerStateEditor=Ext.create('DSS.editor.PowerStateEditorPart',{});
		var pad = function(i) { return (i < 10 ? '0'+i : i); };

        var oConfigName={
            defaultSortCol:'name',
            disabledProperty:'disabled',
            storeConfig:{
                storeId : 'udsStore',
                fields : [ {name:'baseID'}, 
                           {name:'id'}, 
                           {name:'type', sortType:'asUCText'},
                           {name:'name', sortType:'asUCText'},
                           {name:'setName', sortType:'asUCText'},
                           {name:'resetName', sortType:'asUCText'},
                           {name:'showOnPhone', sortType:'asUCText'},
                           {name:'activityState', sortType: 'asUCText'},
                           {name:'combinedState', sortType: 'asUCText'},
                           {name:'sensorState', sortType: 'asUCText'},
                           {name:'powerState', sortType: 'asUCText'},
                           {name:'consumptionState', sortType: 'asUCText'},
                           {name:'windowState', sortType: 'asUCText'},
                           {name:'grouping'}
                           ] 
            },
            multiselect:true,
            grouperField : 'grouping',
            columns:[             
                 {
                     header : _('Name'),
                     dataIndex : 'name',
                     xtype: 'textcolumn',
                     flex:2,
                     customColumnRenderer :  function(val) {
                    	 return Ext.String.htmlEncode(unescape(val));
                     }
                 },
                 {
                     header : _('set description'),
                     dataIndex : 'setName',
                     xtype: 'textcolumn',
                     flex:1,
                     customColumnRenderer :  function(val) {
                     	return '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.baseImagePath + 'state/uds_state_active.png"></td><td>'+ Ext.String.htmlEncode(unescape(val)) +'</td></tr></table>';
                     }
                 },
                {
                    header : _('Trigger to set State'),
                    dataIndex : 'setTrigger',
                    xtype: 'textcolumn',
                    customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                    	if (record.get('type')=='custom-states')
                    		return '<span style="color:#aaaaaa;">'  + _("custom state - triggered manually") + "</span>";
                    	if (record.get('type')=='triggered-states') {
                    		var oRenderer=Ext.create('DSS.component.triggerColumn',{});
                    		return (oRenderer.renderer(record.get('activityState').setTrigger, null, null, 0, 0, null, {up:function(){ return {isFieldActive:function(){return true;}};}} ));
                    	}
                    	if (record.get('type')=='power-states') {
                    		return Ext.String.format2(_('{meter} Meteringvalue > {value} W'),{meter:dss.staticDataModel.getMeterName(record.get('powerState').dsid),value:record.get('powerState').upperValue}) ;
                    	}
                    	if (record.get('type')=='combined-states') {
                    		var sString="<table>";
                    		var oObject;
                    		
                    		var fFirst=true;
                    		if (record.get('combinedState').or!=null) {                    			
                    			oObject=record.get('combinedState').or;
                    			if (oObject.length>1)
                    				sString+='<tr><td rowspan="'+oObject.length+'" style="padding-right:5px">' + _('OR') + '</td>';
                    			else
                    				sString+='<tr><td rowspan="'+oObject.length+'" style="padding-right:5px"></td>';
                    		} else {
                    			oObject=record.get('combinedState').and;
                    			if (oObject.length>1)
                    				sString+='<tr><td rowspan="'+oObject.length+'" style="padding-right:5px">' + _('AND') + '</td>';
                    			else
                    				sString+='<tr><td rowspan="'+oObject.length+'" style="padding-right:5px"></td>';
                    		}
                    		for (var iIndex=0;iIndex<oObject.length;iIndex++) {
                    			if (!fFirst)
                    				sString +="<tr>";
                    			sString+="<td>" + me.combinedStateEditor.getStateString(oObject[iIndex].value,oObject[iIndex].state) + "</td></tr>"; 
                    			fFirst=false;
                    		}
                    		return sString + "</table>";
                    	}
                    	if (record.get('type')=='zone-sensor-states') {
                    		var sString;
                    		var baseObject=record.get('sensorState');
                    		
                    		if (baseObject.zone.greaterOnSet)
                    			sString = "&gt; ";
                			else
                				sString = "&lt; ";
                    		sString += baseObject.zone.activeValue;
                    		switch (baseObject.zone.typeID) {
                    			case 9:
                    			case 10:
                    				sString+='&deg;';
                    			break;
                    			case 11:
                    			case 12:
                    				sString+='Lx';
                    			break;
                    			case 13:
                    			case 14:
                    				sString+='%';
                    			break;         
                    			case 15:
                    				sString+='hPa';
                        		break;
                    			case 18:
                    				sString+='m/s';
                        		break;
                    			case 19:
                    				sString+='&deg;';
                        		break;
                    			case 20:
                    				sString+='mm/m2';
                        		break;
                    			case 21:
                    				sString+='ppm';
                        		break;
                    			case 25:
                    				sString+='dB';
                    			break;
                    		}

                    		return Ext.String.format2(_('{sensortype} for room {zonename} is {condition}'),{
                    			sensortype:dss.staticDataModel.getNameForSensortype(baseObject.zone.typeID),
                    			zonename:dss.staticDataModel.getNameForZone(baseObject.zone.zoneID),
                    			condition:sString
                    		});                    		
                    	}
                    	if (record.get('type')=='consumption-states') {
                    		var oDevice=dss.staticDataModel.getDevice(record.get('consumptionState').dSID);
                    		var eventName=_('Event not defined yet in Hardwaretab');
                    		for (var iIndex=0;iIndex<oDevice.sensorEvents.length;iIndex++) {
                    			if (oDevice.sensorEvents[iIndex].id=='event0')
                    				eventName=oDevice.sensorEvents[iIndex].name;
                    		}
                            var sString = oDevice.name + " " + _("in") + " ";
                            sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                            return '<img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '">&nbsp;' + sString + ':' + eventName;
                    	}
                    	if (record.get('type')=='window-states') {
                    		var oArray=record.get('windowState');
                    		var sString="<table>";
                    		for (var iIndex=0;iIndex<oArray.length;iIndex++) {
                    			var oDevice=dss.staticDataModel.getDevice(oArray[iIndex].dsid);
                    			sString+='<tr><td><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td><td>';
                    			sString+=oDevice.name + " " + _("in") + " " + dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                    			sString+=":</td><td>";
                    			switch(oArray[iIndex].state) {
	                    			case "open":
	                    				sString+=_("should be open");
	                    			break;
	                    			case "open-tilded":
	                    				sString+=_("should be open or tilded");
	                    			break;
	                    			case "closed":
	                    				sString+=_("should be closed");
	                    			break;
	                    			case "closed-tilded":
	                    				sString+=_("should be closed or tilded");
	                    			break;
                    			}
                    			sString+="</td></tr>";                    			
                    		}
                            return sString+'</table>';
                    	}
                    	if (record.get('type')=='device-sensor-states') {
                    		var sString;
                    		var baseObject=record.get('sensorState');
                    		
                    		if (baseObject.device.greaterOnSet)
                    			sString = "&gt; ";
                			else
                				sString = "&lt; ";
                    		sString += baseObject.device.activeValue;
                    		switch (baseObject.device.typeID) {
                    			case 9:
                    			case 10:
                    				sString+='&deg;';
                    			break;
                    			case 11:
                    			case 12:
                    				sString+='Lx';
                    			break;
                    			case 13:
                    			case 14:
                    				sString+='%';
                    			break;         
                    			case 15:
                    				sString+='hPa';
                        		break;
                    			case 18:
                    				sString+='m/s';
                        		break;
                    			case 19:
                    				sString+='&deg;';
                        		break;
                    			case 20:
                    				sString+='mm/m2';
                        		break;
                    			case 21:
                    				sString+='ppm';
                        		break;
                    			case 25:
                    				sString+='dB';
                    			break;
                    		}
                    		var device=dss.staticDataModel.getDevice(baseObject.device.dsid);
                    		return Ext.String.format2(_('{sensortype} from {devicename} in {zoneofdevice} is {condition}'),{
                    			sensortype:dss.staticDataModel.getNameForSensortype(baseObject.device.typeID),
                    			devicename:device.name,
                    			zoneofdevice:dss.staticDataModel.getNameForZone(device.zoneID),
                    			condition:sString
                    		});
                    	}
                    	return "";
                    },
                    flex:3
                },
                {
                    header : _('Period'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width:50,
                    customColumnRenderer :  function(val, metadata, record, rowIdx, colIdx, store, view) {
                    	if (record.get('type')=='triggered-states') {
                    		return "" + pad(Math.floor(record.get('activityState').setRetriggeringLag/60)) + ":" + pad(record.get('activityState').setRetriggeringLag%60);
                    	}
                    	if (record.get('type')=='power-states') {
                    		return "" + pad(Math.floor(record.get('powerState').setRetriggeringLag/60)) + ":" + pad(record.get('powerState').setRetriggeringLag%60);
                    	}
                    	if (record.get('type')=='consumption-states') {
                    		return "" + pad(Math.floor(record.get('consumptionState').setRetriggeringLag/60)) + ":" + pad(record.get('consumptionState').setRetriggeringLag%60);
                    	} 
                    	return '<span style="color:#aaaaaa;">-</span>';
                    }
                },
                {
                    header : _('reset description'),
                    dataIndex : 'resetName',
                    xtype: 'textcolumn',
                    flex:1,
                    customColumnRenderer :  function(val) {
                    	return '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.baseImagePath + 'state/uds_state_inactive.png"></td><td>'+Ext.String.htmlEncode(unescape(val)) +'</td></tr></table>';
                    }
                },
                {
                    header : _('Trigger to reset State'),
                    dataIndex : 'resetTrigger',
                    xtype: 'textcolumn',
                    customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
                    	if (record.get('type')=='custom-states')
                    		return '<span style="color:#aaaaaa;">'  + _("custom state - triggered manually")  + "</span>";
                    	if (record.get('type')=='triggered-states') {
                    		var oRenderer=Ext.create('DSS.component.triggerColumn',{});
                    		return (oRenderer.renderer(record.get('activityState').resetTrigger, null, null, 0, 0, null, {up:function(){ return {isFieldActive:function(){return true;}};}} ));
                    	}
                    	if (record.get('type')=='combined-states') {
                    		return '<span style="color:#aaaaaa;">-</span>';
                    	}
                    	if (record.get('type')=='window-states') {
                    		return '<span style="color:#aaaaaa;">-</span>';
                    	}
                    	if (record.get('type')=='power-states') {
                    		return Ext.String.format2(_('Meteringvalue < {value} W'),{value:record.get('powerState').lowerValue}) ;
                    	}
                    	if (record.get('type')=='zone-sensor-states') {
                    		var sString;
                    		var baseObject=record.get('sensorState');
                    		
                    		if (baseObject.zone.greaterOnSet)
                    			sString = "&lt; ";
                			else
                				sString = "&gt; ";
                    		sString += baseObject.zone.inactiveValue;
                    		switch (baseObject.zone.typeID) {
                    			case 9:
                    			case 10:
                    				sString+='&deg;';
                    			break;
                    			case 11:
                    			case 12:
                    				sString+='Lx';
                    			break;
                    			case 13:
                    			case 14:
                    				sString+='%';
                    			break;         
                    			case 15:
                    				sString+='hPa';
                        		break;
                    			case 18:
                    				sString+='m/s';
                        		break;
                    			case 19:
                    				sString+='&deg;';
                        		break;
                    			case 20:
                    				sString+='mm/m2';
                        		break;
                    			case 21:
                    				sString+='ppm';
                        		break;
                    			case 25:
                    				sString+='dB';
                    			break;
                    		}

                    		return Ext.String.format2(_('{sensortype} for room {zonename} is {condition}'),{
                    			sensortype:dss.staticDataModel.getNameForSensortype(baseObject.zone.typeID),
                    			zonename:dss.staticDataModel.getNameForZone(baseObject.zone.zoneID),
                    			condition:sString
                    		});
                    		
                    	}
                    	if (record.get('type')=='consumption-states') {
                    		var oDevice=dss.staticDataModel.getDevice(record.get('consumptionState').dSID);
                    		var eventName=_('Event not defined yet in Hardwaretab');
                    		for (var iIndex=0;iIndex<oDevice.sensorEvents.length;iIndex++) {
                    			if (oDevice.sensorEvents[iIndex].id=='event1')
                    				eventName=oDevice.sensorEvents[iIndex].name;
                    		}
                            var sString = oDevice.name + " " + _("in") + " ";
                            sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                            return '<img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '">&nbsp;' + sString + ':' + eventName;

                    	}
                    	if (record.get('type')=='device-sensor-states') {
                    		var sString;
                    		var baseObject=record.get('sensorState');
                    		
                    		if (baseObject.device.greaterOnSet)
                    			sString = "&lt; ";
                			else
                				sString = "&gt; ";
                    		sString += baseObject.device.inactiveValue;
                    		switch (baseObject.device.typeID) {
                    			case 9:
                    			case 10:
                    				sString+='&deg;';
                    			break;
                    			case 11:
                    			case 12:
                    				sString+='Lx';
                    			break;
                    			case 13:
                    			case 14:
                    				sString+='%';
                    			break;         
                    			case 15:
                    				sString+='hPa';
                        		break;
                    			case 18:
                    				sString+='m/s';
                        		break;
                    			case 19:
                    				sString+='&deg;';
                        		break;
                    			case 20:
                    				sString+='mm/m2';
                        		break;
                    			case 21:
                    				sString+='ppm';
                        		break;
                    			case 25:
                    				sString+='dB';
                    			break;
                    		}
                    		var device=dss.staticDataModel.getDevice(baseObject.device.dsid);
                    		return Ext.String.format2(_('{sensortype} from {devicename} in {zoneofdevice} is {condition}'),{
                    			sensortype:dss.staticDataModel.getNameForSensortype(baseObject.device.typeID),
                    			devicename:device.name,
                    			zoneofdevice:dss.staticDataModel.getNameForZone(baseObject.device.zoneID),
                    			condition:sString
                    		});
                    	}
                    	return "";
                    },
                    flex:3
                },
                {
                    header : _('Period'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width:50,
                    customColumnRenderer :  function(val, metadata, record, rowIdx, colIdx, store, view) {
                    	if (record.get('type')=='triggered-states') {
                    		return "" + pad(Math.floor(record.get('activityState').resetRetriggeringLag/60)) + ":" + pad(record.get('activityState').resetRetriggeringLag%60);
                    	}
                    	if (record.get('type')=='power-states') {
                    		return "" + pad(Math.floor(record.get('powerState').resetRetriggeringLag/60)) + ":" + pad(record.get('powerState').resetRetriggeringLag%60);
                    	}
                    	if (record.get('type')=='consumption-states') {
                    		return "" + pad(Math.floor(record.get('consumptionState').resetRetriggeringLag/60)) + ":" + pad(record.get('consumptionState').resetRetriggeringLag%60);
                    	}
 
                    	return '<span style="color:#aaaaaa;">-</span>';
                    }
                },
                {
                    header : _('Current state'),
                    dataIndex : 'baseID',
                    xtype: 'textcolumn',
                    customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) { 
                    	
                    	for (var iIndex=0;iIndex<dss.staticDataModel.userStates.length;iIndex++)
                    		if (dss.staticDataModel.userStates[iIndex].name==record.get('baseID')) {
                    			if (dss.staticDataModel.userStates[iIndex].value==1)
                                	return '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.baseImagePath + 'state/uds_state_active.png"></td><td>'+Ext.String.htmlEncode(unescape(record.get('setName'))) +'</td></tr></table>';
                    			if (dss.staticDataModel.userStates[iIndex].value==2)
                                	return '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.baseImagePath + 'state/uds_state_inactive.png"></td><td>'+Ext.String.htmlEncode(unescape(record.get('resetName')))+'</td></tr></table>';
                    		}
                    },
                    flex:1
                },
                {
                    header : _('Show on Phone'),
                    dataIndex : 'showOnPhone',
                    xtype: 'phonecolumn',
                    width:50,
                    listeners: {
                    	checkPhoneChange : function (column,record,dataIndex, checked) {
                    		if (checked)
	                            me.waitAndReload(me.oNamePanel,{
	                                actions : 'setShowOnPhone',
	                                id : record.get('id')
	                            });
                    		else
	                            me.waitAndReload(me.oNamePanel,{
	                                actions : 'resetShowOnPhone',
	                                id : record.get('id')
	                            });
                    	}
                    }
                	
                }
            ],
            buttonText: {
                newButton: _('New state'),
                editButton: _('Edit state'),
                testButton:_('Toggle state'),
                deleteButton: _('Delete state')
            },
            editorConfig:{
                title : _('Edit user defined state'),
                staticControls : [  {
                    type : 'hidden',
                    label : 'name',
                    id : 'name'
                }, {
                    type : 'hidden',
                    label : 'showOnPhone',
                    id : 'showOnPhone'
                }, {
                    type : 'custom',
                    id : 'type',
                    control: me.starterStateEditor
                }, {
                    type : 'hidden',
                    label : 'id',
                    id : 'setName'
                }, {
                    type : 'hidden',
                    label : 'id',
                    id : 'resetName'
                }, {
                    type : 'hidden',
                    label : 'id',
                    id : 'id'
                }, {
                	type: 'custom',
                	id: 'combinedNames',
                	control : me.nameEditor 
                }, {
                	type: 'custom',
                	id: 'activityState',
                	control : me.activityStateEditor 
                }, {
                	type: 'custom',
                	id: 'combinedState',
                	control : me.combinedStateEditor 
                }, {
                	type: 'custom',
                	id: 'sensorState',
                	control : me.sensorStateEditor 
                }, {
                	type: 'custom',
                	id: 'consumptionState',
                	control : me.consumptionStateEditor 
                }, {
                	type: 'custom',
                	id: 'powerState',
                	control : me.powerStateEditor 
                }, {
                	type: 'custom',
                	id: 'windowState',
                	control : me.windowStateEditor 
                }
                
                
                ]
            },
            listeners:{
                'loadDataStore':function (grid){
                    me.loadParameter(grid);
                },
                'saveEntry':function (grid,values){                        
                    me.waitAndReload(grid,{
                        actions : 'save',
                        value : Ext.JSON.encode(values)
                    });
                },
                'deleteEntry':function (grid,entry){
                	var idVal;
                	if (entry.get!=null) {
                		idVal=entry.get('id');
                	} else {
                		idVal=[];
                		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                			if (entry[iIndex]!=null)
                				idVal.push(entry[iIndex].get('id'));
                	}

                    me.waitAndReload(grid, {
                        actions : 'delete',
                        id : Ext.JSON.encode(idVal)
                    });
                },
                'testEntry':function (grid,entry) {
                	var oID=entry.get('baseID');
                	for (var iIndex=0;iIndex<dss.staticDataModel.userStates.length;iIndex++) {
                		if (dss.staticDataModel.userStates[iIndex].name==oID) {
                			dss.ajaxSyncRequest('/json/state/set', {
                				addon : "system-addon-user-defined-states",
                				name : oID,
                				value : (dss.staticDataModel.userStates[iIndex].value==1)?'inactive':'active'
                			});
                		}
                	}
                    window.setTimeout(function() {
                        me.loadParameter(grid);
                    }, 2 * 1000);
                },
                'selectionChanged':function (grid,values) {
                	me.oNamePanel.btnTest.setDisabled(true);
                	if (values.length==1) {
                		if (values[0].get('type')=='custom-states')
                			me.oNamePanel.btnTest.setDisabled(false);
                		if (values[0].get('type')=='consumption-states')
                			me.oNamePanel.btnTest.setDisabled(false);
                		if (values[0].get('type')=='triggered-states')
                			me.oNamePanel.btnTest.setDisabled(false);
                	}
                }                
            }
        };                
        this.oNamePanel = Ext.create('DSS.gridPanel', oConfigName);
        
        return this.oNamePanel;
    },
    waitAndReload : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
    	
    	var me = this;
    	dss.pushAndQuery('system-addon-user-defined-states.config',parameter,
    			         'system-addon-user-defined-states.saved',function(eventData) {
    		 me.loadParameter(grid);
    		 return true;
    	}); 
               
    },
    loadUDS : function () {
    	dss.staticDataModel.initUserStateCache();
    	var sString = dss.ajaxSyncRequest('/json/property/query2', {
            'query' : '/scripts/system-addon-user-defined-states/*(*)/*(*)/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        var oArray=[];
        for (var sTypeKey in result.result) {
        	if (['consumption-states','triggered-states','combined-states','zone-sensor-states','device-sensor-states','window-states','power-states','custom-states'].indexOf(sTypeKey)>-1)
	    		for (var subKey in result.result[sTypeKey]) {
	            	var resultObject=result.result[sTypeKey][subKey];
	            	var grouping=sTypeKey;
	            	if (grouping=='device-sensor-states')
	            		grouping='sensor-states';
	            	if (grouping=='zone-sensor-states')
	            		grouping='sensor-states';
	            	var oObject={
	            		baseID:subKey,
	            		id:resultObject.id,
	            		name:escape(dss.unescapeProperty(resultObject.name)),
	            		setName:escape(dss.unescapeProperty(resultObject.setName)),
	            		resetName:escape(dss.unescapeProperty(resultObject.resetName)),
	            		showOnPhone:resultObject.showOnPhone,
	            		type:sTypeKey,
	            		grouping:_(grouping)
	            	};
	            	
	            	if (resultObject.completeName!=null)
	            		oObject.baseID=resultObject.completeName;
	            	switch (sTypeKey) {
	            		case "consumption-states": {
	            			oObject.consumptionState={
	            				dSID:resultObject.setTrigger.triggers['0'].dsuid,
	            				setRetriggeringLag:0,
	            				resetRetriggeringLag:0
	            			};
	            			if (resultObject.setTrigger.damping!=null)
	            				if (resultObject.setTrigger.damping.interval!=null)
	                    			oObject.consumptionState.setRetriggeringLag=resultObject.setTrigger.damping.interval;
	            			if (resultObject.resetTrigger.damping!=null)
	            				if (resultObject.resetTrigger.damping.interval!=null)
	                    			oObject.consumptionState.resetRetriggeringLag=resultObject.resetTrigger.damping.interval;
	            		}
	            		break;
	            		case "triggered-states":
	            			var setTriggers=[];
	            			if (resultObject.setTrigger!=null)
	            			for (var sKey in resultObject.setTrigger.triggers) {
	            				if (resultObject.setTrigger.triggers[sKey].type=='addon-state-change')
	            					if (resultObject.setTrigger.triggers[sKey]['addon-id']!=null)
	            						resultObject.setTrigger.triggers[sKey].addonId=resultObject.setTrigger.triggers[sKey]['addon-id'];
	            				setTriggers.push(resultObject.setTrigger.triggers[sKey]);
	            				
	            			}
	            			var resetTriggers=[];
	            			if (resultObject.resetTrigger!=null)
		            			for (var sKey in resultObject.resetTrigger.triggers) {
	            				if (resultObject.resetTrigger.triggers[sKey].type=='addon-state-change')
	            					if (resultObject.resetTrigger.triggers[sKey]['addon-id']!=null)
	            						resultObject.resetTrigger.triggers[sKey].addonId=resultObject.resetTrigger.triggers[sKey]['addon-id'];
	            				resetTriggers.push(resultObject.resetTrigger.triggers[sKey]);
	            			}
	            			oObject.activityState={
	            					setTrigger:setTriggers,
	            					resetTrigger:resetTriggers,
	                				setRetriggeringLag:0,
	                				resetRetriggeringLag:0
	            			};
	            			if (resultObject.setTrigger!=null)
		            			if (resultObject.setTrigger.damping!=null)
		            				if (resultObject.setTrigger.damping.interval!=null)
		                    			oObject.activityState.setRetriggeringLag=resultObject.setTrigger.damping.interval;
	            			if (resultObject.resetTrigger!=null)
		            			if (resultObject.resetTrigger.damping!=null)
		            				if (resultObject.resetTrigger.damping.interval!=null)
		                    			oObject.activityState.resetRetriggeringLag=resultObject.resetTrigger.damping.interval;
	            		break;
	            		case "combined-states":
	            			var fOr=(resultObject.resetTrigger.conditions!=null);
	            			var oArraySub=[];
	            			for (var sKey in resultObject.setTrigger.triggers) {
	            				var oObjectSub={
	            					state:resultObject.setTrigger.triggers[sKey].name,
	            					value:resultObject.setTrigger.triggers[sKey].state,
	            					addonid:''
	            				};
	            				if (resultObject.setTrigger.triggers[sKey].type=='addon-state-change') {
	            					oObjectSub.addonid=resultObject.setTrigger.triggers[sKey]['addon-id'];
	            				}
	            				oArraySub.push(oObjectSub);
	            			}
	            			if (fOr)
	            				oObject.combinedState={or:oArraySub};
	            			else
	            				oObject.combinedState={and:oArraySub};
	            		break;
	            		case "zone-sensor-states":
	            			oObject.sensorState={
	            				zone:{
		            				zoneID:resultObject.source.zone,
		            				groupID:resultObject.source.group,
		            				typeID:resultObject.source.type
	            			}};
	            			if (resultObject.activeValue.substr(0,4)=='&gt;')
	            				oObject.sensorState.zone.greaterOnSet=true;
	            			else
	            				oObject.sensorState.zone.greaterOnSet=false;
	            			oObject.sensorState.zone.activeValue=parseFloat(resultObject.activeValue.substr(7));
	            			oObject.sensorState.zone.inactiveValue=parseFloat(resultObject.inactiveValue.substr(7));
	                	break;
	            		case "device-sensor-states":
	            			oObject.sensorState={
	            				device:{
	            					dsid:resultObject.source.dsid,
	            					typeID:resultObject.source.type            					
	            			}};
	            			if (resultObject.activeValue.substr(0,4)=='&gt;')
	            				oObject.sensorState.device.greaterOnSet=true;
	            			else
	            				oObject.sensorState.device.greaterOnSet=false;
	            			oObject.sensorState.device.activeValue=parseFloat(resultObject.activeValue.substr(7));
	            			oObject.sensorState.device.inactiveValue=parseFloat(resultObject.inactiveValue.substr(7));
	                	break;
	            		case "window-states":
	            			oObject.windowState=[];
	            			for (var sKey in resultObject.setTrigger.triggers) {
	            				var dsid=resultObject.setTrigger.triggers[sKey].name.split('_')[0];
	            				oObject.windowState.push({
	        						dsid:resultObject.setTrigger.triggers[sKey].name.split('_')[0],
	        						state:resultObject.setTrigger.triggers[sKey].name.split('_')[1]
	        					});
	            			}
	            		break;
	            		case "power-states": {
	            			oObject.powerState={
	            				upperValue:parseInt(""+resultObject.activeValue,10),
	            				lowerValue:parseInt(""+resultObject.inactiveValue,10),
	            				dsid:resultObject.source.dsid,
	            				setRetriggeringLag:0,
	            				resetRetriggeringLag:0
	            			}
	            			if (resultObject.setTrigger.damping!=null)
	            				if (resultObject.setTrigger.damping.interval!=null)
	                    			oObject.powerState.setRetriggeringLag=resultObject.setTrigger.damping.interval;
	            			if (resultObject.resetTrigger.damping!=null)
	            				if (resultObject.resetTrigger.damping.interval!=null)
	                    			oObject.powerState.resetRetriggeringLag=resultObject.resetTrigger.damping.interval;
	            		}
	            	}
	            	oArray.push(oObject);
	    		}
        }
        return oArray;
    },
    loadParameter : function(grid) {
    	grid.setStatus(_('Loading ...'));
    	dss.staticDataModel.initUserStateCache();
        var oResultObject=this.loadUDS();
        if (oResultObject!=null) {
            grid.loadParameter(oResultObject);
        }
        grid.setStatus(_('Ready'));
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/system-addon-user-defined-states.po']);
    var HLEEdit = Ext.create('DSS.addon.UDSEdit');
    HLEEdit.initPage();
});
